#ifndef QEMU_HYPERV_VMBUS_PAGE_MAP_H
#define QEMU_HYPERV_VMBUS_PAGE_MAP_H

#include "config-host.h"
#include "sysemu/sysemu.h"

/**
 * Guest PFN slot mapping info
 */
typedef struct hv_page_map_slot {
    uint64_t pfn;   /* Guest PFN */
    void *address;  /* Mapped host VA, or NULL */
    bool rw;        /* true if mapped for writing */
} hv_page_map_slot;

/**
 * A simple guest PFN page map
 *
 * Guest allocates a range of PFNs for underlying channel ringbuffer
 * and shares it with host through VMBus.
 */
typedef struct hv_page_map {
    uint32_t nr_pages;
    hv_page_map_slot pg_slot[0];
} hv_page_map;

hv_page_map *hv_page_map_create(const uint64_t *pfn_array, uint32_t nr_pages);

void hv_page_map_destroy(hv_page_map *map);

void *hv_page_map_map(hv_page_map *map, uint32_t pg_index, bool rw);

void hv_page_map_unmap(hv_page_map *map, uint32_t pg_index);

bool hv_page_map_read(hv_page_map *map, uint32_t pos, void *dst, uint32_t len);

bool hv_page_map_write(hv_page_map *map, uint32_t pos, void *src, uint32_t len);

void hv_page_map_save(hv_page_map *map, QEMUFile *f);

hv_page_map *hv_page_map_load(QEMUFile *f);

#endif
