/*
 *
 * Copyright (c) 2011, Microsoft Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307 USA.
 *
 * Authors:
 *   Haiyang Zhang <haiyangz@microsoft.com>
 *   Hank Janssen  <hjanssen@microsoft.com>
 *   K. Y. Srinivasan <kys@microsoft.com>
 *
 */

/*
 * VMBus ring buffer API
 * Ported from MS LIS
 */

#ifndef QEMU_HYPERV_VMBUS_RINGBUFFER_H
#define QEMU_HYPERV_VMBUS_RINGBUFFER_H

#include "target-i386/hyperv_defs.h"

struct hv_page_map;

struct hv_kvec {
    void *iov_base;
    size_t iov_len;
};

struct hv_ring_buffer_info {
    struct hv_ring_buffer *ring_buffer;
    uint32_t ring_size;         /* Include the shared header */

    uint32_t ring_datasize;     /* < ring_size */
    uint32_t ring_data_startoffset;

    struct hv_page_map *pmap;
};


struct hv_ring_buffer_debug_info {
    uint32_t current_interrupt_mask;
    uint32_t current_read_index;
    uint32_t current_write_index;
    uint32_t bytes_avail_toread;
    uint32_t bytes_avail_towrite;
};

int hv_ringbuffer_init(struct hv_ring_buffer_info *ring_info,
        const uint64_t *pfn_array, uint32_t nr_pages);

void hv_ringbuffer_cleanup(struct hv_ring_buffer_info *ring_info);

int hv_ringbuffer_write(struct hv_ring_buffer_info *ring_info,
            struct hv_kvec *kv_list,
            uint32_t kv_count, bool *signal);

int hv_ringbuffer_peek(struct hv_ring_buffer_info *ring_info, void *buffer,
           uint32_t buflen);

int hv_ringbuffer_read(struct hv_ring_buffer_info *ring_info,
           void *buffer,
           uint32_t buflen,
           uint32_t offset, bool *signal);

void hv_ringbuffer_get_debuginfo(struct hv_ring_buffer_info *ring_info,
                struct hv_ring_buffer_debug_info *debug_info);

void hv_begin_read(struct hv_ring_buffer_info *rbi);

uint32_t hv_end_read(struct hv_ring_buffer_info *rbi);

#endif
