/*
 * QEMU KVM Hyper-V support
 *
 * Copyright (C) 2015 Andrey Smetanin <asmetanin@virtuozzo.com>
 *
 * Authors:
 *  Andrey Smetanin <asmetanin@virtuozzo.com>
 *
 * This work is licensed under the terms of the GNU GPL, version 2 or later.
 * See the COPYING file in the top-level directory.
 *
 */

#ifndef HYPERV_I386_H
#define HYPERV_I386_H

#include "cpu.h"
#include "sysemu/kvm.h"
#include "qemu/event_notifier.h"

typedef struct HvSintRoute HvSintRoute;
typedef void (*HvSintAckClb)(HvSintRoute *sint_route);

struct HvSintRoute {
    /*< private >*/
    Object parent_obj;
    /*< public >*/
    uint32_t sint;
    X86CPU *cpu;
    int gsi;
    EventNotifier sint_set_notifier;
    EventNotifier sint_ack_notifier;
    HvSintAckClb sint_ack_clb;
    int refcnt;

    struct hv_message msg;
    /*
     * the state of the message staged in .msg:
     * 0        - the staging area is not in use (after init or message
     *            successfully delivered to guest)
     * -EBUSY   - the staging area is being used in vcpu thread
     * -EAGAIN  - delivery attempt failed due to slot being busy, retry
     * -EXXXX   - error
     */
    int msg_status;
};

int kvm_hv_handle_exit(X86CPU *cpu, struct kvm_hyperv_exit *exit);

HvSintRoute *hyperv_sint_route_new(X86CPU *cpu, uint32_t sint,
                                   const char *name,
                                   HvSintAckClb sint_ack_clb);

HvSintRoute *hyperv_sint_route_get(X86CPU *cpu, const char *name);
void hyperv_sint_route_unref(HvSintRoute *sint_route);

int kvm_hv_sint_route_set_sint(HvSintRoute *sint_route);

X86CPU *hyperv_find_vcpu(uint32_t vcpu_id);

void hyperv_post_msg(HvSintRoute *sint_route, struct hv_message *msg);

int hyperv_set_evt_flag(HvSintRoute *sint_route, unsigned evtno);

void hyperv_add_synic_props(X86CPU *cpu);

static inline bool hyperv_is_fast_hcall(struct kvm_hyperv_exit *exit) {
    return (exit->u.hcall.input >> 16) & 0x1;
}

#endif
